<?php

namespace Drupal\Tests\webform_ct\FunctionalJavascript;

use Drupal\webform\WebformInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\webform\Traits\WebformBrowserTestTrait;

/**
 * Checks escaping for custom webform_ct functionality with javascript enabled.
 *
 * @group webform_ct_tests
 */
class WebformCtFunctionalJavascriptTest extends WebDriverTestBase {
  use WebformBrowserTestTrait;

  /**
   * An admin user with all permissions.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * User without 'administer_webform_confirmation_javascript' permission.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'webform',
    'webform_ct',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->createAdminRole('administrator', 'administrator');
    $this->adminUser = $this->createUser([]);
    $this->adminUser->addRole('administrator');
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests if code in the javascript textarea is not escaped.
   */
  public function testAlertJavascriptTextareaNotEscaped() {
    /**
     * @var \Behat\Mink\Driver\Selenium2Driver $driver
     */
    $driver = $this->getSession()->getDriver();
    // Create Webform:
    $webform = $this->createWebform();
    $webformId = $webform->getOriginalId();
    // Set the webform settings with the alert:
    $webform->setSetting('confirmation_type', WebformInterface::CONFIRMATION_PAGE);
    $webform->setThirdPartySetting('webform_ct', 'confirmation_custom_javascript', "localStorage.setItem('testkey', 'testname');");
    $webform->save();
    // Go to confirmation page and see if the script got escaped:
    $this->drupalGet('/webform/' . $webformId . '/confirmation');
    $lsValue = $driver->getWebDriverSession()->local_storage()->getKey('testkey');
    $this->assertEquals('testname', $lsValue);
  }

}
